@@ -1,34 +1,77 @@
+## This function is equivalent to:
+##    fun <- asFunction(expr, envir=envir, ...)
+##    codetools::findGlobals(fun, merge=TRUE)
+## but we expand it here to make it more explicit
+## what is going on.
+#' @importFrom codetools makeUsageCollector findLocalsList walkCode
 findGlobals_conservative <- function(expr, envir, ...) {
-  fun <- asFunction(expr, envir=envir, ...)
-  names <- codetools::findGlobals(fun, merge=TRUE)
-  names
-}
-
-findGlobals_liberal <- function(expr, envir, ...) {
-  requireNamespace("codetools")
-
-  vars <- new.env(hash=TRUE, parent=emptyenv())
-  funs <- new.env(hash=TRUE, parent=emptyenv())
+  objs <- character()
 
   enter <- function(type, v, e, w) {
-    if (type == "function")
-      assign(v, value=TRUE, envir=funs)
-    else
-      assign(v, value=TRUE, envir=vars)
+    objs <<- c(objs, v)
+  }
+
+  ## From codetools::findGlobals():
+  fun <- asFunction(expr, envir=envir, ...)
+  # codetools::collectUsage(fun, enterGlobal=enter)
+
+  ## The latter becomes equivalent to (after cleanup):
+  w <- makeUsageCollector(fun, enterGlobal=enter, name="<anonymous>")
+  w$env <- new.env(hash=TRUE, parent=w$env)
+  locals <- findLocalsList(list(expr))
+  for (name in locals) assign(name, value=TRUE, envir=w$env)
+  walkCode(expr, w)
+
+  unique(objs)
+}
+
+
+#' @importFrom codetools makeUsageCollector walkCode
+findGlobals_liberal <- function(expr, envir, ...) {
+  objs <- character()
+
+  enter <- function(type, v, e, w) {
+    objs <<- c(objs, v)
   }
 
   fun <- asFunction(expr, envir=envir, ...)
-  w <- codetools::makeUsageCollector(fun, enterGlobal=enter, name="<anonymous>")
-  codetools::walkCode(expr, w)
+  w <- makeUsageCollector(fun, enterGlobal=enter, name="<anonymous>")
+  walkCode(expr, w)
 
-  fnames <- ls(funs, all.names = TRUE)
-  vnames <- ls(vars, all.names = TRUE)
-  sort(unique(c(fnames, vnames)))
+  unique(objs)
+}
+
+
+#' @importFrom codetools makeUsageCollector walkCode
+findGlobals_ordered <- function(expr, envir, ...) {
+  class <- name <- character()
+
+  enterLocal <- function(type, v, e, w) {
+    class <<- c(class, "local")
+    name <<- c(name, v)
+  }
+
+  enterGlobal <- function(type, v, e, w) {
+    class <<- c(class, "global")
+    name <<- c(name, v)
+  }
+
+  fun <- asFunction(expr, envir=envir, ...)
+  w <- makeUsageCollector(fun, name="<anonymous>",
+                          enterLocal=enterLocal, enterGlobal=enterGlobal)
+  walkCode(expr, w)
+
+  ## Drop duplicated names
+  dups <- duplicated(name)
+  class <- class[!dups]
+  name <- name[!dups]
+
+  unique(name[class == "global"])
 }
 
 
 #' @export
-findGlobals <- function(expr, envir=parent.frame(), ..., tweak=NULL, dotdotdot=c("warning", "error", "return", "ignore"), method=c("conservative", "liberal"), substitute=FALSE, unlist=TRUE) {
+findGlobals <- function(expr, envir=parent.frame(), ..., tweak=NULL, dotdotdot=c("warning", "error", "return", "ignore"), method=c("ordered", "conservative", "liberal"), substitute=FALSE, unlist=TRUE) {
   method <- match.arg(method)
   dotdotdot <- match.arg(dotdotdot)
 
@@ -56,10 +99,12 @@ findGlobals <- function(expr, envir=parent.frame(), ..., tweak=NULL, dotdotdot=c
 
   if (is.function(tweak)) expr <- tweak(expr)
 
-  if (method == "liberal") {
-    findGlobalsT <- findGlobals_liberal
-  } else {
+  if (method == "ordered") {
+    findGlobalsT <- findGlobals_ordered
+  } else if (method == "conservative") {
     findGlobalsT <- findGlobals_conservative
+  } else if (method == "liberal") {
+    findGlobalsT <- findGlobals_liberal
   }
 
   ## Is there a need for global '...' variables?
