@@ -2,12 +2,12 @@
 #'
 #' @param expr An R expression.
 #' @param envir The environment from where to search for globals.
-#' @param ... Not used.
+#' @param \dots Not used.
 #' @param method A character string specifying what type of search algorithm to use.
 #' @param tweak An optional function that takes an expression
 #'        and returns a tweaked expression.
 ## @param dotdotdot A @character string specifying how to handle a
-##        \emph{global} \code{...} if one is discovered.
+##        \emph{global} \code{\dots} if one is discovered.
 #' @param substitute If TRUE, the expression is \code{substitute()}:ed,
 #'        otherwise not.
 #' @param mustExist If TRUE, an error is thrown if the object of the
@@ -19,7 +19,11 @@
 #' @return A \link{Globals} object.
 #'
 #' @details
-#' There currently two methods for identifying global objects.
+#' There currently three methods for identifying global objects.
+#'
+#' The \code{"ordered"} search method identifies globals such that
+#' a global variable preceeding a local variable with the same name
+#' is not dropped (which the \code{"conservative"} method would).
 #'
 #' The \code{"conservative"} search method tries to keep the number
 #' of false positive to a minimum, i.e. the identified objects are
@@ -42,7 +46,7 @@
 #'
 #' @aliases findGlobals
 #' @export
-globalsOf <- function(expr, envir=parent.frame(), ..., method=c("conservative", "liberal"), tweak=NULL, substitute=FALSE, mustExist=TRUE, unlist=TRUE) {
+globalsOf <- function(expr, envir=parent.frame(), ..., method=c("ordered", "conservative", "liberal"), tweak=NULL, substitute=FALSE, mustExist=TRUE, unlist=TRUE) {
   method <- match.arg(method)
 
   if (substitute) expr <- substitute(expr)
